@@ -2,30 +2,10 @@ use std::fmt::{Debug, Formatter};
 use std::fmt;
 
 use crate::bits;
-use crate::errors::QCompressError;
+use crate::bits::{LEFT_MASKS, RIGHT_MASKS};
+use crate::errors::{QCompressError, QCompressResult};
 use crate::types::UnsignedLike;
 
-const LEFT_MASKS: [u8; 8] = [
-  0xff,
-  0x7f,
-  0x3f,
-  0x1f,
-  0x0f,
-  0x07,
-  0x03,
-  0x01,
-];
-const RIGHT_MASKS: [u8; 8] = [
-  0x00,
-  0x80,
-  0xc0,
-  0xe0,
-  0xf0,
-  0xf8,
-  0xfc,
-  0xfe,
-];
-
 #[derive(Clone)]
 pub struct BitReader {
   bytes: Vec<u8>,
@@ -73,7 +53,7 @@ impl BitReader {
     }
   }
 
-  pub fn read_bytes(&mut self, n: usize) -> Result<&[u8], QCompressError> {
+  pub fn read_bytes(&mut self, n: usize) -> QCompressResult<&[u8]> {
     self.refresh_if_needed();
 
     if self.j == 0 {
@@ -164,7 +144,7 @@ impl BitReader {
     res
   }
 
-  pub fn drain_bytes(&mut self) -> Result<&[u8], QCompressError> {
+  pub fn drain_bytes(&mut self) -> QCompressResult<&[u8]> {
     if self.j != 0 {
       self.i += 1;
       self.j = 0;
@@ -172,15 +152,35 @@ impl BitReader {
     let n = self.bytes.len() - self.i;
     self.read_bytes(n)
   }
+
+  // Seek to the end of the byte.
+  // Used to skip to the next metadata or body section of the file, since they
+  // always start byte-aligned.
+  pub fn drain_byte(&mut self) {
+    self.j = 8;
+  }
+
+  pub fn seek_bytes(&mut self, n_bytes: usize) {
+    self.j = 8;
+    self.i += n_bytes;
+  }
+
+  pub fn inds(&self) -> (usize, usize) {
+    (self.i, self.j)
+  }
+
+  pub fn size(&self) -> usize {
+    self.bytes.len()
+  }
 }
 
 #[cfg(test)]
 mod tests {
   use crate::BitReader;
-  use crate::errors::QCompressError;
+  use crate::errors::QCompressResult;
 
   #[test]
-  fn test_bit_reader() -> Result<(), QCompressError>{
+  fn test_bit_reader() -> QCompressResult<()>{
     // bits: 1001 1010  0110 1011  0010 1101
     let bytes = vec![0x9a, 0x6b, 0x2d];
     let mut bit_reader = BitReader::from(bytes);
