@@ -1,15 +1,18 @@
 use std::cmp::{max, min};
 use std::fmt;
 use std::fmt::Debug;
+use std::marker::PhantomData;
 
+use crate::{BitWriter, Flags, huffman};
 use crate::bits::*;
-use crate::huffman;
+use crate::chunk_metadata::ChunkMetadata;
+use crate::constants::*;
+use crate::errors::{QCompressError, QCompressResult};
 use crate::prefix::{Prefix, PrefixIntermediate};
 use crate::types::{NumberLike, UnsignedLike};
 use crate::utils;
-use crate::constants::*;
-use crate::errors::QCompressError;
 
+const DEFAULT_MAX_DEPTH: u32 = 6;
 const MIN_N_TO_USE_RUN_LEN: usize = 1001;
 const MIN_FREQUENCY_TO_USE_RUN_LEN: f64 = 0.8;
 
@@ -19,10 +22,10 @@ struct JumpstartConfiguration {
 }
 
 fn choose_run_len_jumpstart(
-  weight: u64,
+  count: u64,
   n: u64,
 ) -> JumpstartConfiguration {
-  let freq = (weight as f64) / (n as f64);
+  let freq = (count as f64) / (n as f64);
   let non_freq = 1.0 - freq;
   let jumpstart = min((-non_freq.log2()).ceil() as usize, MAX_JUMPSTART);
   let expected_n_runs = (freq * non_freq * n as f64).ceil() as u64;
@@ -32,7 +35,7 @@ fn choose_run_len_jumpstart(
   }
 }
 
-fn push_pref<T: Copy>(
+fn push_pref<T: NumberLike>(
   seq: &mut Vec<PrefixIntermediate<T>>,
   prefix_idx: &mut usize,
   i: usize,
@@ -41,40 +44,63 @@ fn push_pref<T: Copy>(
   n: usize,
   sorted: &[T],
 ) {
-  let weight = j - i;
-  let frequency = weight as f64 / n as f64;
+  let count = j - i;
+  let frequency = count as f64 / n as f64;
   let new_prefix_idx = max(*prefix_idx + 1, (j * max_n_prefix) / n);
   let prefix_idx_incr = new_prefix_idx - *prefix_idx;
-  if n < MIN_N_TO_USE_RUN_LEN || frequency < MIN_FREQUENCY_TO_USE_RUN_LEN || weight == n || prefix_idx_incr == 1 {
+  if n < MIN_N_TO_USE_RUN_LEN || frequency < MIN_FREQUENCY_TO_USE_RUN_LEN || count == n || prefix_idx_incr == 1 {
     // The usual case - a prefix for a range that represents either 100% or
     // <=80% of the data.
-    // This also works if there are no other ranges.
-    seq.push(PrefixIntermediate::new(weight as u64, sorted[i], sorted[j - 1], None));
+    seq.push(PrefixIntermediate::new(
+      count,
+      count as u64,
+      sorted[i],
+      sorted[j - 1],
+      None
+    ));
   } else {
     // The weird case - a range that represents almost all (but not all) the data.
     // We create extra prefixes that can describe `reps` copies of the range at once.
-    let config = choose_run_len_jumpstart(weight as u64, n as u64);
-    seq.push(PrefixIntermediate::new(config.weight, sorted[i], sorted[j - 1], Some(config.jumpstart)));
+    let config = choose_run_len_jumpstart(count as u64, n as u64);
+    seq.push(PrefixIntermediate::new(
+      count,
+      config.weight,
+      sorted[i],
+      sorted[j - 1],
+      Some(config.jumpstart)
+    ));
   }
   *prefix_idx = new_prefix_idx;
 }
 
-#[derive(Clone)]
-pub struct Compressor<T> where T: NumberLike {
-  prefixes: Vec<Prefix<T>>,
-  n: usize,
+#[derive(Clone, Debug)]
+pub struct CompressorConfig {
+  pub max_depth: u32,
 }
 
-impl<T: 'static> Compressor<T> where T: NumberLike {
-  pub fn train(nums: Vec<T>, max_depth: u32) -> Result<Self, QCompressError> {
+impl Default for CompressorConfig {
+  fn default() -> Self {
+    Self {
+      max_depth: DEFAULT_MAX_DEPTH,
+    }
+  }
+}
+
+#[derive(Clone, Default)]
+struct TrainedChunkCompressor<T> where T: NumberLike {
+  prefixes: Vec<Prefix<T>>,
+}
+
+impl<T> TrainedChunkCompressor<T> where T: NumberLike + 'static {
+  pub fn train(nums: Vec<T>, config: CompressorConfig, _flags: Flags) -> QCompressResult<Self> {
+    let max_depth = config.max_depth;
     if max_depth > MAX_MAX_DEPTH {
       return Err(QCompressError::MaxDepthError { max_depth });
     }
     let n = nums.len();
     if n == 0 {
-      return Ok(Compressor::<T> {
-        prefixes: Vec::new(),
-        n,
+      return Ok(TrainedChunkCompressor::<T> {
+        ..Default::default()
       });
     }
     if n as u64 > MAX_ENTRIES {
@@ -131,6 +157,7 @@ impl<T: 'static> Compressor<T> where T: NumberLike {
         let pref0 = &prefix_sequence[best_i as usize];
         let pref1 = &prefix_sequence[best_i as usize + 1];
         prefix_sequence[best_i as usize] = PrefixIntermediate::new(
+          pref0.count + pref1.count,
           pref0.weight + pref1.weight,
           pref0.lower,
           pref1.upper,
@@ -144,45 +171,47 @@ impl<T: 'static> Compressor<T> where T: NumberLike {
     huffman::make_huffman_code(&mut prefix_sequence);
 
     let mut prefixes = Vec::new();
-    for p in &prefix_sequence {
-      prefixes.push(Prefix::from_intermediate_and_diff(p));
+    for p in prefix_sequence {
+      prefixes.push(Prefix::from(p));
     }
 
-    let res = Compressor::<T> {
+    let res = TrainedChunkCompressor::<T> {
       prefixes,
-      n,
     };
     Ok(res)
   }
 
-  pub fn combine_improvement(p0: &PrefixIntermediate<T>, p1: &PrefixIntermediate<T>, n: usize) -> f64 {
+  fn combine_improvement(p0: &PrefixIntermediate<T>, p1: &PrefixIntermediate<T>, n: usize) -> f64 {
     if p0.run_len_jumpstart.is_some() || p1.run_len_jumpstart.is_some() {
       // can never combine prefixes that encode run length
       return f64::MIN;
     }
 
-    let p0_r_cost = avg_base2_bits(T::offset_diff(p0.upper, p0.lower));
-    let p1_r_cost = avg_base2_bits(T::offset_diff(p1.upper, p1.lower));
-    let combined_r_cost = avg_base2_bits(T::offset_diff(p1.upper, p0.lower));
+    let p0_r_cost = avg_base2_bits(p0.upper.to_unsigned() - p0.lower.to_unsigned());
+    let p1_r_cost = avg_base2_bits(p1.upper.to_unsigned() - p1.lower.to_unsigned());
+    let combined_r_cost = avg_base2_bits(p1.upper.to_unsigned() - p0.lower.to_unsigned());
     let p0_d_cost = depth_bits(p0.weight, n);
     let p1_d_cost = depth_bits(p1.weight, n);
     let combined_d_cost = depth_bits(p0.weight + p1.weight, n);
-    let meta_cost = 10.0 + 2.0 * T::PHYSICAL_BITS as f64;
+    let meta_cost = 10.0 +
+      BITS_TO_ENCODE_N_ENTRIES as f64 +
+      2.0 * T::PHYSICAL_BITS as f64;
 
     let separate_cost = 2.0 * meta_cost +
       (p0_r_cost + p0_d_cost) * p0.weight as f64+
       (p1_r_cost + p1_d_cost) * p1.weight as f64;
     let combined_cost = meta_cost +
       (combined_r_cost + combined_d_cost) * (p0.weight + p1.weight) as f64;
-    let bits_saved = separate_cost - combined_cost;
-    bits_saved as f64
+
+    separate_cost - combined_cost
   }
 
-  fn compress_num_offset_bits_w_prefix(&self, num: T, pref: &Prefix<T>, v: &mut Vec<bool>) {
-    let off = T::offset_diff(num, pref.lower);
-    extend_with_diff_bits(off, pref.k, v);
+  fn compress_num_offset_bits_w_prefix(&self, num: T, pref: &Prefix<T>, writer: &mut BitWriter) {
+    let off = num.to_unsigned() - pref.lower_unsigned;
+    writer.write_diff(off, pref.k);
     if off < pref.only_k_bits_lower || off > pref.only_k_bits_upper {
-      v.push((off & (T::Diff::ONE << pref.k)) > T::Diff::ZERO) // most significant bit, if necessary, comes last
+      // most significant bit, if necessary, comes last
+      writer.write_one((off & (T::Unsigned::ONE << pref.k)) > T::Unsigned::ZERO);
     }
   }
 
@@ -190,36 +219,34 @@ impl<T: 'static> Compressor<T> where T: NumberLike {
     num.ge(&prefix.lower) && num.le(&prefix.upper)
   }
 
-  pub fn compress_nums_as_bits(&self, nums: &[T]) -> Result<Vec<bool>, QCompressError> {
+  fn compress_nums(&self, nums: &[T], writer: &mut BitWriter) -> QCompressResult<()> {
     let mut sorted_prefixes = self.prefixes.clone();
-    // most reps comes first
+    // most common prefixes come first
     sorted_prefixes.sort_by(
       |p0, p1|
-        p0.val.len().cmp(&p1.val.len())
+        p0.count.cmp(&p1.count)
     );
 
-    let mut res = Vec::new();
-    let res_ptr = &mut res;
     let mut i = 0;
     while i < nums.len() {
       let mut success = false;
       let num = nums[i];
       for pref in &sorted_prefixes {
-        if !Self::in_prefix(num, &pref) {
+        if !Self::in_prefix(num, pref) {
           continue;
         }
 
-        res_ptr.extend(&pref.val);
+        writer.write_bits(&pref.val);
 
         match pref.run_len_jumpstart {
           None => {
-            self.compress_num_offset_bits_w_prefix(num, &pref, res_ptr);
+            self.compress_num_offset_bits_w_prefix(num, pref, writer);
             i += 1;
           }
           Some(jumpstart) => {
             let mut reps = 1;
             for other_num in nums.iter().skip(i + 1) {
-              if Self::in_prefix(*other_num, &pref) {
+              if Self::in_prefix(*other_num, pref) {
                 reps += 1;
               } else {
                 break;
@@ -228,10 +255,10 @@ impl<T: 'static> Compressor<T> where T: NumberLike {
 
             // we store 1 less than the number of occurrences
             // because the prefix already implies there is at least 1 occurrence
-            res_ptr.extend(usize_to_varint_bits(reps - 1, jumpstart));
+            writer.write_varint(reps - 1, jumpstart);
 
             for x in nums.iter().skip(i).take(reps) {
-              self.compress_num_offset_bits_w_prefix(*x, &pref, res_ptr);
+              self.compress_num_offset_bits_w_prefix(*x, pref, writer);
             }
             i += reps;
           }
@@ -247,47 +274,84 @@ impl<T: 'static> Compressor<T> where T: NumberLike {
         });
       }
     }
-    Ok(res)
+    writer.finish_byte();
+    Ok(())
   }
 
-  pub fn metadata_as_bits(&self) -> Vec<bool> {
-    let mut res = Vec::new();
-    res.extend(bytes_to_bits(MAGIC_HEADER.to_vec()));
-    res.extend(&byte_to_bits(T::HEADER_BYTE));
-    res.extend(usize_to_bits(self.n, BITS_TO_ENCODE_N_ENTRIES));
-    res.extend(usize_to_bits(self.prefixes.len(), MAX_MAX_DEPTH));
-    for pref in &self.prefixes {
-      res.extend(bytes_to_bits(T::bytes_from(pref.lower)));
-      res.extend(bytes_to_bits(T::bytes_from(pref.upper)));
-      res.extend(usize_to_bits(pref.val.len(), BITS_TO_ENCODE_PREFIX_LEN));
-      res.extend(&pref.val);
-      match pref.run_len_jumpstart {
-        None => {
-          res.push(false);
-        },
-        Some(jumpstart) => {
-          res.push(true);
-          res.extend(usize_to_bits(jumpstart, BITS_TO_ENCODE_JUMPSTART))
-        },
-      }
-    }
-    res
-  }
+  fn compress_chunk(&self, nums: &[T], writer: &mut BitWriter) -> QCompressResult<ChunkMetadata<T>> {
+    writer.write_aligned_byte(MAGIC_CHUNK_BYTE)?;
+    let pre_header_idx = writer.size();
+    let mut metadata = ChunkMetadata {
+      n: nums.len(),
+      // temporarily write compressed body size as 0; we'll fix this after
+      // actually writing the compressed body and figuring out how big it is
+      compressed_body_size: 0,
+      prefixes: self.prefixes.clone()
+    };
+    metadata.write_to(writer);
 
-  pub fn compress_as_bits(&self, nums: &[T]) -> Result<Vec<bool>, QCompressError> {
-    let mut compression = self.metadata_as_bits();
-    let mut num_bits = self.compress_nums_as_bits(nums)?;
-    compression.append(&mut num_bits);
-    Ok(compression)
-  }
+    let post_header_idx = writer.size();
+    self.compress_nums(nums, writer)?;
 
-  pub fn compress(&self, nums: &[T]) -> Result<Vec<u8>, QCompressError> {
-    Ok(bits_to_bytes(self.compress_as_bits(nums)?))
+    metadata.compressed_body_size = writer.size() - post_header_idx;
+    metadata.update_write_compressed_body_size(writer, pre_header_idx);
+    Ok(metadata)
   }
 }
 
-impl<T> Debug for Compressor<T> where T: NumberLike {
+impl<T> Debug for TrainedChunkCompressor<T> where T: NumberLike {
   fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     utils::display_prefixes(&self.prefixes, f)
   }
 }
+
+#[derive(Clone, Debug, Default)]
+pub struct Compressor<T> where T: NumberLike {
+  pub config: CompressorConfig,
+  pub flags: Flags,
+  pub phantom: PhantomData<T>,
+}
+
+impl<T> Compressor<T> where T: NumberLike + 'static {
+  pub fn from_config(config: CompressorConfig) -> Self {
+    Self {
+      config,
+      ..Default::default()
+    }
+  }
+
+  pub fn from_config_and_flags(config: CompressorConfig, flags: Flags) -> Self {
+    Self {
+      config,
+      flags,
+      ..Default::default()
+    }
+  }
+
+  pub fn header(&self, writer: &mut BitWriter) -> QCompressResult<()> {
+    writer.write_aligned_bytes(&MAGIC_HEADER)?;
+    writer.write_aligned_byte(T::HEADER_BYTE)?;
+    self.flags.write(writer)
+  }
+
+  pub fn compress_chunk(&self, nums: &[T], writer: &mut BitWriter) -> QCompressResult<ChunkMetadata<T>> {
+    let chunk_compressor = TrainedChunkCompressor::train(
+      nums.to_vec(),
+      self.config.clone(),
+      self.flags.clone(),
+    )?;
+    chunk_compressor.compress_chunk(nums, writer)
+  }
+
+  pub fn footer(&self, writer: &mut BitWriter) -> QCompressResult<()> {
+    writer.write_aligned_byte(MAGIC_TERMINATION_BYTE)
+  }
+
+  pub fn simple_compress(&self, nums: &[T]) -> QCompressResult<Vec<u8>> {
+    let mut writer = BitWriter::default();
+    self.header(&mut writer)?;
+    self.compress_chunk(nums, &mut writer)?;
+    self.footer(&mut writer)?;
+    Ok(writer.pop())
+  }
+}
