@@ -1,10 +1,14 @@
 pub const MAGIC_HEADER: [u8; 4] = [113, 99, 111, 33]; // ascii for qco!
-pub const MAX_ENTRIES: u64 = (1_u64 << 32) - 1;
-pub const BITS_TO_ENCODE_N_ENTRIES: u32 = 32;
+pub const MAGIC_CHUNK_BYTE: u8 = 44; // ,
+pub const MAGIC_TERMINATION_BYTE: u8 = 46; // .
+
+pub const MAX_ENTRIES: u64 = (1_u64 << 24) - 1;
+pub const BITS_TO_ENCODE_N_ENTRIES: u32 = 24;
 pub const MAX_MAX_DEPTH: u32 = 15;
 pub const BITS_TO_ENCODE_PREFIX_LEN: u32 = 4;
 pub const MAX_JUMPSTART: usize = 31;
 pub const BITS_TO_ENCODE_JUMPSTART: u32 = 5;
+pub const BITS_TO_ENCODE_COMPRESSED_BODY_SIZE: u32 = 32;
 
 #[cfg(test)]
 mod tests {
